#include "sysconfig.h"
#include "sysdeps.h"

#include "uae.h"

#include <camd.h>

#ifdef CAMD


#include <alsa/asoundlib.h>
#include <stdbool.h>
#include <stdint.h>


#define NUM_DEVS 8

static snd_seq_t *camd_seq_in[NUM_DEVS];
static snd_seq_t *camd_seq_out[NUM_DEVS];
static int camd_queue[NUM_DEVS];


// Code for create_alsa_seq made by looking at jack-rack by bob ham.
static snd_seq_t *create_alsa_seq(const char *client_name,int dev,bool isinput){
  snd_seq_t * seq;
  int err;
  
  err = snd_seq_open(&seq, "default", SND_SEQ_OPEN_DUPLEX, 0);
  if (err){
    printf( "Could not open ALSA sequencer, aborting\n\n%s\n\n"
	    "Make sure you have configure ALSA properly and that\n"
	    "/proc/asound/seq/clients exists and contains relevant\n"
	    "devices.\n", snd_strerror (err));
    return NULL;
  }
  
  snd_seq_set_client_name (seq, client_name);
  
  err = snd_seq_create_simple_port (seq, isinput==true?"Input":"Output",
                                    (isinput==true?SND_SEQ_PORT_CAP_WRITE:SND_SEQ_PORT_CAP_READ)|SND_SEQ_PORT_CAP_DUPLEX|
                                    SND_SEQ_PORT_CAP_SUBS_READ|SND_SEQ_PORT_CAP_SUBS_WRITE,
                                    SND_SEQ_PORT_TYPE_APPLICATION|SND_SEQ_PORT_TYPE_SPECIFIC);
  if (err){
    printf ( "Could not create ALSA port, aborting\n\n%s\n", snd_strerror (err));
    snd_seq_close(seq);
    return NULL;
  }

  if(isinput==false){
    camd_queue[dev] = snd_seq_alloc_queue(seq); 
    snd_seq_start_queue(seq, camd_queue[dev], NULL );
  }

  return seq;
}


static void alsaseq_PutMidi(
			    int dev,
			    uint32_t msg
			    )
{
  unsigned int d3=msg&0xff;
  unsigned int d2=(msg>>8)&0xff;
  unsigned int d1=(msg>>16)&0xff;

  unsigned char buffer[3]={d1,d2,d3};
  snd_seq_event_t ev;
  snd_midi_event_t *midi_ev;
  snd_midi_event_new( 10, &midi_ev );
  
  //printf("sending %x %x %x to dev %d\n",d1,d2,d3,dev);
  snd_seq_ev_clear( &ev );
  snd_midi_event_encode(midi_ev,
			buffer,
			3,
			&ev); 
  snd_midi_event_free( midi_ev );
  
  snd_seq_ev_set_source(&ev,0);
  snd_seq_ev_set_subs(&ev);
  
  snd_seq_ev_schedule_tick( &ev, camd_queue[dev], 1, 1);
  
  snd_seq_event_output(camd_seq_out[dev], &ev);
  
  snd_seq_drain_output(camd_seq_out[dev]);

}



void camd_Init(void){
  int lokke;
  for(lokke=0;lokke<NUM_DEVS;lokke++){
    char temp[500];
    sprintf(temp,"Uae%d",lokke);
    camd_seq_in[lokke]=create_alsa_seq(temp,lokke,true);
    camd_seq_out[lokke]=create_alsa_seq(temp,lokke,false);
  }
}


uae_u32 camd_PutMidi(uae_u32 dev, uae_u32 data)
{
  alsaseq_PutMidi(dev,data);
  //printf("camd_PutMidi dev: %d data: %x\n",dev,data);
  return 0 ;
}


static uae_u32 camd_decode(int val1, int val2, int val3){
  return val1<<24|val2<<16|val3<<8;
}


extern struct pollfd das_poll[1];
extern int lastactive;
uae_u32 camd_GetMidi(uae_u32 dev)
{
  snd_seq_event_t *event;

  if(dev==0){
    static int c=0;
    c++;
    if(c>4){
      das_poll[0].fd=-1;
      das_poll[0].events = POLLIN|POLLERR|POLLHUP|POLLNVAL;
      //printf("1\n");
      poll(das_poll,1,1);
      lastactive++;
      c=0;
    }
  }

  //printf("1 camd_GetMidi dev: %d\n",device);
  if(snd_seq_event_input_pending(camd_seq_in[dev],1)){
    snd_seq_event_input(camd_seq_in[dev], &event);
    //printf("got something\n");
    switch(event->type){
    case SND_SEQ_EVENT_NOTEON:
      //printf("Noteon, channel: %d note: %d vol: %d\n",event->data.note.channel,event->data.note.note,event->data.note.velocity);
      return camd_decode(0x90+event->data.note.channel,event->data.note.note,event->data.note.velocity);
      break;
    case SND_SEQ_EVENT_NOTEOFF:
      //printf("Noteoff, channel: %d note: %d vol: %d\n",event->data.note.channel,event->data.note.note,event->data.note.velocity);
      return camd_decode(0x90+event->data.note.channel,event->data.note.note,0);
      break;
    case SND_SEQ_EVENT_KEYPRESS:
      //printf("Keypress, channel: %d note: %d vol: %d\n",event->data.note.channel,event->data.note.note,event->data.note.velocity);
      return camd_decode(0xa0+event->data.note.channel,event->data.note.note,event->data.note.velocity);
      break;
    case SND_SEQ_EVENT_CONTROLLER:
      //printf("Control: %d %d %d\n",event->data.control.channel,event->data.control.param,event->data.control.value);
      return camd_decode(0xb0+event->data.control.channel,event->data.control.param,event->data.control.value);
      break;
    case SND_SEQ_EVENT_PITCHBEND:
      //printf("Pitch: %d %d %d\n",event->data.control.channel,event->data.control.param,event->data.control.value);
      {
	int val=event->data.control.value + 0x2000;
	return camd_decode(0xe0+event->data.control.channel,val&127,val>>7);
      }
      break;
    case SND_SEQ_EVENT_CHANPRESS:
      //printf("chanpress: %d %d %d\n",event->data.control.channel,event->data.control.param,event->data.control.value);
      return camd_decode(0xc0+event->data.control.channel,event->data.control.value,0);
      break;
    case SND_SEQ_EVENT_PGMCHANGE:
      // printf("pgmchange: %d %d %d\n",event->data.control.channel,event->data.control.param,event->data.control.value);
      return camd_decode(0xc0+event->data.control.channel,event->data.control.value,0);
      break;
    case SND_SEQ_EVENT_START:
      return camd_decode(0xfa,0,0);
      break;
    case SND_SEQ_EVENT_CONTINUE:
      return camd_decode(0xfb,0,0);
      break;
    case SND_SEQ_EVENT_STOP:
      return camd_decode(0xfc,0,0);
      break;
      //case SND_SEQ_EVENT_SETPOS_TICK:
      //return camd_decode(0xf,0,0);
      //break;
      //case SND_SEQ_EVENT_SETPOS_TIME:
      //return camd_decode(0xf,0,0);
      //break;
      //case SND_SEQ_EVENT_TEMPO:
      //return camd_decode(0xf,0,0);
      //break;
      //case SND_SEQ_EVENT_CLOCK:
      //return camd_decode(0xf,0,0);
      //break;
      //case SND_SEQ_EVENT_TICK: 
      //return camd_decode(0xf,0,0);
      //break;
    case SND_SEQ_EVENT_RESET:
      return camd_decode(0xff,0,0);
      break;
    case SND_SEQ_EVENT_SENSING:
      return camd_decode(0xfe,0,0);
      break;
    default:
      printf("uae_camd/Can't handle type: %d\n",event->type);
      break;
    }

  }
  
  //printf("2 camd_GetMidi dev: %d\n",device);
  return 0;
}




#endif

